
import processing.opengl.*;

// exploration with visual scale
// inspired by "powers of ten"
// start with relatively macro view
// then push increasingly closer
// then into an individual pixel
// and into the underlying numbers and bits which comprise it.

// 2008 robert carlsen - robertcarlsen.net

// start simple:
// smoothly zoom into a rectangle. ok.
// create a scale and use that to zoom. ok. needs refinement.

static final float MIN_ZOOM = 0.75;
static final float MAX_ZOOM = 800.0;
static final float CONTROL_EXPONENT = 4f; // adjusts the exponential scale factor in the sidebar

static final float ZOOM_TARGETS[] = {
  MIN_ZOOM,90,MAX_ZOOM};

Integrator zoom = new Integrator(MIN_ZOOM);
Integrator pos[] = new Integrator[2];

// for on screen text;
PFont font;

// load in specific images
PImage wideImg;
PImage twentyImg;
PImage fourhundredImg;

// scale scroller:
ScaleBox scroller;

boolean intro = true;

void setup() {
  size(800,600, OPENGL);

  font = createFont("SansSerif",10);
  textMode(MODEL);

  wideImg = loadImage("macbook_top.jpg");
  twentyImg = loadImage("20x_alpha.png");
  fourhundredImg = loadImage("400x_alpha.png");

  ortho(-width/2, width/2, -height/2, height/2, -10000, 10000); 

  scroller = new ScaleBox(this);

  // default zoom/position 
  zoom.set(0); 
  pos[0] = new Integrator(width/2);
  pos[1] = new Integrator(height/2);

}


void draw() {
  background(20);
  smooth();

  if(intro){
    zoom.target(ZOOM_TARGETS[0]);
    pos[0].target(585);
    pos[1].target(315);
    intro=false;
  }

  // for the smooth moving
  zoom.update();
  pos[0].update();
  pos[1].update();

  pushMatrix();
  translate(pos[0].value,pos[1].value,0);
  scale(zoom.value);

  rectMode(CENTER);
  //fill(255);
  //rect(0,0,100,100);


  //image(wideImg,-wideImg.width/2,-wideImg.height/2);
  // percentages derived in photoshop:
  image(wideImg,-wideImg.width*.738,-wideImg.height*.527);

  // the scale is cumulative, so we only need to adjust by the relative amount between each image's scale level.
  // these values were derived by measuring the pixel sizes for common elements in each image.
  // 1:75 for the wide to 20x image,
  // 75:1100 (~1:14) for the 20x to 400x image (although these images are different sizes, too)

  scale(1/55f);
  translate(-32,-18); // aligning things
  image(twentyImg,-twentyImg.width/2,-twentyImg.height/2);
  //image(twentyImg,0,0);

  scale(1/14f);
  translate(450,250); // aligning things
  image(fourhundredImg,-fourhundredImg.width/2,-fourhundredImg.height/2);

  popMatrix();

  // draw the scale control:
  //drawScale();
  scroller.draw();
}


void mousePressed() {
  handleMouse();
}

void mouseDragged() {
  handleMouse();
}


void mouseReleased() {
  //if(mouseX>30)
  //  zoom.target(1); 

  println(pos[0].value+","+pos[1].value);
  println(zoom.value);
}

void handleMouse() {
  if(mouseX<30){
    // log scale:
    //float currentValue = map(mouseY,10,height-10,1,100);
    //float inputScale = map(log(currentValue),0,4.6,MAX_ZOOM,MIN_ZOOM);

    // exponential:
    float currentValue = constrain(map(mouseY,scroller.margin,height-scroller.margin,0.001,1.0),0.001,1.0);
    float inputScale = map(pow(currentValue,1/CONTROL_EXPONENT),0,1,MAX_ZOOM,MIN_ZOOM);

    // linear scale:
    //float inputScale = constrain(map(mouseY,10,height-10,MAX_ZOOM,MIN_ZOOM),MIN_ZOOM,MAX_ZOOM);

    zoom.target(inputScale);
  }
  else{
    //    zoom.target(1);
    pos[0].target(pos[0].value-(pmouseX-mouseX)*10f);
    pos[1].target(pos[1].value-(pmouseY-mouseY)*10f);
  } 
}


void keyPressed() {
  switch(key){
  case '1':
    /*
    zoom.target(0.5);
     pos[0].target(369);
     pos[1].target(250);
     */
    zoom.target(ZOOM_TARGETS[0]);
    pos[0].target(585);
    pos[1].target(315);
    break;
  case '2':
    zoom.target(ZOOM_TARGETS[1]);
    pos[0].target(455);
    pos[1].target(330);
    break;
  case '3':
    zoom.target(ZOOM_TARGETS[2]);
    pos[0].target(430);
    pos[1].target(290);
    break;
  case '-':
    float dec = zoom.value > (MIN_ZOOM+10) ? zoom.value-10:zoom.value;
    zoom.target(dec);
    break;

  case '=':
  case '+':
    float inc = zoom.value < MAX_ZOOM-10 ? zoom.value+10:zoom.value;
    zoom.target(inc);
    break;

  } 

}


class ScaleBox {
  PApplet parent;
  float margin = 10;

  float width;
  float height;

  ScaleBox(PApplet p) {
    parent = p;
    width = 15;
    height = parent.height-(margin*2);

  }

  void draw() {
    pushMatrix();
    translate(margin,margin,MAX_ZOOM+1); 
    scale(1f);
    // draw the background:
    fill(240,150);
    noStroke();
    rectMode(CORNER);
    rect(0,0,width,height);

    stroke(50);
    fill(50);

    // draw the vertical mark for the current scale:
    // linear scale:
    //float currentScale = map(zoom.value,MAX_ZOOM,MIN_ZOOM,0,scaleBox[1]);

    // log scale:
    //float currentValue = norm(zoom.value,MAX_ZOOM,MIN_ZOOM)*10;
    //currentValue = log(currentValue);
    //float currentScale = map(currentValue,1,2,0,scaleBox[1]);

    // exponential:
    float currentValue = map(zoom.value,MAX_ZOOM,MIN_ZOOM,0.001,1.0);
    float currentScale = map(pow(currentValue,CONTROL_EXPONENT),0,1,0,height);

    line(0,currentScale,width,currentScale);
    float linesY = currentScale;
    while(linesY<=height){
      line(0, linesY,width, linesY);
      linesY+=2;
    }


    // draw some labels:
    textFont(font,10);
    /*
  textAlign(LEFT);
     text(MIN_ZOOM,0,scaleHeight+margin);
     textAlign(RIGHT);
     text(MAX_ZOOM,scaleBoxWidth,scaleHeight+margin);
     */

    // indicate the current scale:
    textAlign(LEFT);

    // percent is nice, but it doesn't really imform us
    // really need something to anchor the values
    // maybe also a scale in psyical units
    // float percent = norm(zoom.value,MIN_ZOOM,MAX_ZOOM); 
    // text((int)(100*percent) + "%",scaleBox[0]+5,currentScale);

    // going to fake it?
    // 1x, 20x and 400x zoom levels?
    float fakeScale = norm(zoom.value,MIN_ZOOM,MAX_ZOOM);
    //fakeScale = map(pow(fakeScale,1.616),0,1,1,400); // this is a real fake, matches the two preset leves but nothing else
    fakeScale = map(fakeScale,0,1,1,400);

    fill(255,200);
    text(nf(fakeScale,0,1) +"x",width+6,currentScale+1); //shadow
    fill(40);
    text(nf(fakeScale,0,1) +"x",width+5,currentScale);

    // draw labels to indicate shortcuts:

    textAlign(RIGHT);
    for(int i=0;i<ZOOM_TARGETS.length;i++){
      float curVal = ZOOM_TARGETS[i]; 
      float pad = -2;
      
      currentValue = map(curVal,MAX_ZOOM,MIN_ZOOM,0.001,1.0);
      currentScale = map(pow(currentValue,CONTROL_EXPONENT),0,1,0,height);

      stroke(255,255,0);
      line(0,currentScale,width,currentScale);
      fill(20,200);
      text(i+1,pad+1,currentScale+1);
      fill(255,255,10);
      text(i+1,pad,currentScale);

    }

    popMatrix();
  }


}

